@@ -88,24 +88,23 @@ class Config:
 class Panda(QtGui.QMainWindow): #main class
 	def __init__( self, parent=None ):
 		QtGui.QWidget.__init__(self, parent)
-		self.ui = None 
-		self.authenticated = False
 		self.c = Config() 
 		self.c.load()
-		self.player = RadioPlayer()
-		self.player.format=self.c.format
-		self.player.start()
+		self.s = snooze ( 0 )
+		self.stimer = QTimer()
+		self.manualPause = False #track manual pausing
+		self.curStation = 0 #Currently playing station
 		self.ui = Ui_pyRadio()
 		self.ui.setupUi(self)
-		self.portrait = False
+		self.stations = None
+		self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbup_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(100, 100, 100);")
+		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
+		self.player = RadioPlayer()
+		self.player.start()
 		self.os = self.player.get_product()
 		if self.os != "RX-51": #check for N900 
 			self.ui.albumArt.setStyleSheet("background-color: rgb(255, 255, 255);")
-
-		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
-
-		if not self.c.lastplayed == "": #auto-play last station
-			self.player.playStation = self.c.lastplayed
+		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Pandora is loading... </font></td><tr></table>" )
 		try : #test for rotation and set auto-rotation on
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
 			self.connect(QApplication.desktop(), SIGNAL("resized(int)"), self.orientationChanged );
@@ -116,53 +115,241 @@ class Panda(QtGui.QMainWindow): #main class
 				self.portrait = True
 			print" Auto rotation on"
 		except:print "could not autorotate on"
-		self.skip = 0
-		self.s = snooze(0)
-		self.createStations = []
-		self.count = 0
+		QtGui.QApplication.processEvents()
+		self.player.format = self.c.format
+		if not self.c.lastplayed == "": #auto-play last station
+			self.autoPlay = self.c.lastplayed
+		else:
+			self.autoPlay = 0 
 		self.connect_signals() #called every time orientation changes to connect slots
+		self.stimer.singleShot(100, self.auth)
+		self.doFM(True)
+
+#Radio Functions
+	def doPlayPause( self ): #Play if stopped, pause if playing
+		self.manualPause = True
+		if self.player.state == 2:
+			self.player.startPlayBack()
+			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+		elif self.player.state == 1:
+			self.player.stopPlayBack()
+			self.ui.btnPlay.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+
+	def handleStates( self ): #receive signals from Backend if changed from somewhere else (ie: headphones)
+		if self.manualPause: #Don't want to overlap with calling doPlayPause
+			self.manualPause = False
+			return
+		if self.player.state == 2:
+			self.player.stopPlayBack()
+			self.ui.btnPlay.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+		elif self.player.state == 1:
+			self.player.startPlayBack()
+			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+
+	def doLove( self ): #Love this song
+		self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
+		"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
+		self.stimer.singleShot(100, self.loveCall)
+		
+	def loveCall( self ):
+		if not self.player.loveHate( self.song, True ):
+			self.ui.btnLove.setStyleSheet("background-color: rgb(100, 100, 100);\n"
+			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
+		else:
+			print "Thumbs up successful."
+
+	def doHate( self ): #Hate this song
+		if self.player.loveHate( self.song, False ):
+			self.playNextSong()
+
+	def tiredSong( self ): #Mark song tired
+		res = self.player.doTired( self.song )
+		if res:
+			self.playNextSong()
+		print res
+
+	def deleteStation( self ):
+		reply = QtGui.QMessageBox.question(self, 'Delete Station',
+			"Really delete "+self.ui.listStations.currentText()+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+		if reply == QtGui.QMessageBox.Yes:
+			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+			self.doPlayPause()
+			self.player.doDelete(self.curStation)
+			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Station Deleted.  Select new station to begin.</font></td><tr></table>" )
+			self.getStations()
+
+	def auth( self, count=0 ): #Start authentication process.
+		if count>2:
+			self.closeEvent()
+		count += 1
+		ok = True #In case we got the user/pass from config
+		if self.c.user == "":
+			self.c.user,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:", QLineEdit.Normal)
+		if not ok: #Username was cancelled.. they must be done.
+			self.closeEvent()
+		if self.c.password == "": 
+			self.c.password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:", QLineEdit.Password)
+		if not ok: #Password was cancelled.. they must be done.
+			self.closeEvent()
+		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authenticating to Pandora as "+self.c.user+" ... </font></td><tr></table>" )
+		QtGui.QApplication.processEvents()
+		authenticated = self.player.auth( self.c.user, self.c.password )
+		if not authenticated:
+			self.auth( count )
+		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication successful.  Select a station from list to the right.</font></td><tr></table>" )
+		QtGui.QApplication.processEvents()
+		self.c.write()
+		self.getStations()
+
+	def playStation( self ): #Play currently selected station
+		stationId = self.stations[self.ui.listStations.currentIndex()]['stationId']	
+		self.curStation = stationId
+		self.player.playlist = []
+		self.player.getMoreSongs( self.curStation, self.c.format )
+		self.playNextSong()
+		if stationId != self.c.lastplayed:
+			f = open(os.path.expanduser("~")+'/.config/pyradio/lastplayed', 'w')
+			f.write(self.curStation)
+			f.close()
+
+	def getStations( self ): #Get Station List	
+		self.ui.listStations.clear()
+		self.stations = self.player.getStations()
+		for s in self.stations: #Stations need to be scanned for QuickMix seperately
+			if s["isQuickMix"]:
+				qMixIds = s["quickMixStationIds"]
+				break
+
+		for s in self.stations: #Add stations
+			#print s
+			#print ""
+			sName = s["stationName"]
+			for qMixId in qMixIds:
+				if qMixId == s["stationId"]:
+					sName += " *" #If we're in the QuickMix list we add a *
+			self.ui.listStations.addItem(sName)
+			if self.autoPlay>0 and s["stationId"] == self.autoPlay:
+				self.autoPlay = 0
+				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
+				self.playStation()
+
+			if self.curStation>0 and s["stationId"] == self.curStation:
+				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
+
+	def setUserPass( self ): #Reset username/password and start authenticating
+		self.c.user = ""
+		self.c.password = ""
 		self.auth()
+
+	def playNextSong( self ): #Go to next song in playlist
+		QtGui.QApplication.processEvents()
+		if not self.player.playlist == []:
+			if self.player.state == 1:
+				self.doPlayPause()
+			print "Getting next song.."
+			self.song = self.player.playlist.pop( 0 )
+			self.setUi( self.song )
+			self.stimer.singleShot(100, lambda: self.player.play(self.song[0], self.song))
+			if self.player.playlist == []:
+				self.stimer.singleShot(500, lambda: self.player.getMoreSongs(self.curStation, self.c.format))
+		else:
+			self.player.getMoreSongs(self.curStation, self.c.format)
+			self.stimer.singleShot(100, self.playNextSong)
+	
+	def setUi ( self, next=None ): #Update the album graphic and set Love graphic
+		art = next[1]["art"]
+		self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
+		self.ui.albumArt.setHtml("<table width='100%' height='100%'><tr><td vertical-align='middle' style='text-align: center'><img style='width: 80%; height: 100%' src='"+art+"'></td></tr></table>")
+		self.ui.lblSong.setText("\"" + next[1]["title"] + "\"\nArtist: " + next[1]["artist"])
+		if next[1]["rating"] > 0:
+			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color + ";\n"
+			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
+		else:
+			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbup_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(100, 100, 100);")
+
+		self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/Play_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: "+ self.c.color + ";")
+
+	def stop( self ): #Stop Playing, not currently implemented
+		print "Temp"
+
+	def closeEvent(self, event=None): #Exit application gracefully.. somewhat.
+		self.player.run = 0
+		self.player.close()
+		del self.ui
+		del self.player
+		quit()
+
+#UI Functions
+	def nextDown(self): #change button color
+		self.ui.btnNext.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")
+
+	def nextUp(self): #change button color
+		self.ui.btnNext.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")
+	
+	def hateDown(self): #change button color
+		self.ui.btnHate.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
+
+	def hateUp(self): #change button color
+		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
+	
+	def setDuration( self, pos, total ):
+		if (total-pos) < total:
+			position = "%02d:%02d" % divmod(total - pos, 60) + " / " + "%02d:%02d" % divmod(total, 60)
+			self.ui.lblTime.setText(position + "\nAlbum: " + self.song[1]["album"])
+
+#Feature Functions
+
+	def setSnooze(self): #Set tired
+		ktime,ok = QtGui.QInputDialog.getText(self,"Snooze", "Enter a time, in minutes, to shutdown:")
+		if ok:
+			self.s.cont = 1
+			self.s.ktime = int(ktime)
+			self.s.start()
+
+	def doFM(self, initial=False): #Turn on FM transmitter
 		try : #test for FM transmitter.
 			sysbus = dbus.SystemBus()
-			self.fmtx = sysbus.get_object('com.nokia.FMTx', '/com/nokia/fmtx/default', False)
-			self.fmtx_iface = dbus.Interface(self.fmtx, dbus_interface='org.freedesktop.DBus.Properties')
+			fmtx = sysbus.get_object('com.nokia.FMTx', '/com/nokia/fmtx/default', False)
+			fmtx_iface = dbus.Interface(fmtx, dbus_interface='org.freedesktop.DBus.Properties')
 
-			state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
-			if state == 'enabled':
-				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+			state = fmtx_iface.Get("com.nokia.FMTx.Device", "state")
+			if state == 'disabled':
+				if not initial: 
+					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+					fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'enabled', variant_level=1))
+				else:
+					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 			else:
-				self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+				if not initial: 
+					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
+					fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))
+				else:
+					self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
 		except: print "No FM Transmitter."
 
+#Orientation Control
+
 	def orientationChanged ( self ): #called when N900 is rotated
 		screenGeometry=QApplication.desktop().screenGeometry();
+		self.disconnect_signals()
 		if screenGeometry.width() > screenGeometry.height(): #landscape
 			self.ui = Ui_pyRadio()
 			self.ui.setupUi(self)
 			self.connect_signals()
 			self.portrait = False
-			if not self.player.curSong == 0:
-				self.setUi ( self.player.curSong )
-				self.buildPlaylist()
 		else: #portrait
 			self.ui = Ui_pyRadio_portrait()
 			self.ui.setupUi(self)
 			self.connect_signals()
 			self.portrait = True
-			if not self.player.curSong == 0:
-				self.setUi ( self.player.curSong )
-				self.buildPlaylist()
 		print "Orientation Changed"
 		self.ui.lblTime.setWordWrap( True )
+		self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbup_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(100, 100, 100);")
+		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
+		self.setUi( self.song )
+		self.stimer.singleShot(100, self.getStations)
 
 	def connect_signals( self ): #connect all signals
-		if not self.authenticated:
-			self.ui.btnPlay.setEnabled( False )
-			self.ui.listStations.setEnabled( False )
-			self.ui.btnLove.setEnabled( False )
-			self.ui.btnHate.setEnabled( False )
-			self.ui.btnSleep.setEnabled( False )
-			self.ui.btnNext.setEnabled( False )
 		QtCore.QObject.connect(self.ui.listStations, QtCore.SIGNAL('activated(int)'), self.playStation)
 		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('pressed()'), self.nextDown)
 		QtCore.QObject.connect(self.ui.btnNext, QtCore.SIGNAL('released()'), self.nextUp)
@@ -173,34 +360,185 @@ class Panda(QtGui.QMainWindow): #main class
 		QtCore.QObject.connect(self.ui.btnPlay, QtCore.SIGNAL('clicked()'), self.doPlayPause)
 		QtCore.QObject.connect(self.ui.btnLove, QtCore.SIGNAL('clicked()'), self.doLove)
 		QtCore.QObject.connect(self.ui.btnSleep, QtCore.SIGNAL('clicked()'), self.tiredSong)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("PlayNext"), self.playNextSong)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("LoveHate"), self.sigLoveHate)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("stations"), self.getStations)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("deleted"), self.stationDeleted)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("tired"), self.sigTired)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("authSuccess"), self.sigAuth)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("authFail"), self.sigAuth)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("stationCreated"), self.stationCreated)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("duration"), self.setDuration)
-		QtCore.QObject.connect(self.player,QtCore.SIGNAL("state(QString)"), self.doPlayPause)
+		QtCore.QObject.connect(self.player,QtCore.SIGNAL("EOF"), self.playNextSong)
+		QtCore.QObject.connect(self.player,QtCore.SIGNAL("state_change"), self.handleStates)
+		QtCore.QObject.connect(self.player,QtCore.SIGNAL("duration(int,int)"), self.setDuration)
 		QtCore.QObject.connect(self.s, QtCore.SIGNAL('shutdown'), self.closeEvent)
 		QtCore.QObject.connect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self.deleteStation)
 		QtCore.QObject.connect(self.ui.actionEnable_FM_Radio, QtCore.SIGNAL('triggered()'), self.doFM)
 		QtCore.QObject.connect(self.ui.actionSnooze, QtCore.SIGNAL('triggered()'), self.setSnooze)
-		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.createStation)
+		QtCore.QObject.connect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.searchStation)
 		QtCore.QObject.connect(self.ui.actionDonate, QtCore.SIGNAL('triggered()'), self.open_donate)
 		QtCore.QObject.connect(self.ui.actionAdd_Seed, QtCore.SIGNAL('triggered()'), self.doSeed)
 		QtCore.QObject.connect(self.ui.actionSettings, QtCore.SIGNAL('triggered()'), self.openSettings)
 
+	def disconnect_signals( self ): #disconnect all signals
+		QtCore.QObject.disconnect(self.ui.listStations, QtCore.SIGNAL('activated(int)'), self.playStation)
+		QtCore.QObject.disconnect(self.ui.btnNext, QtCore.SIGNAL('pressed()'), self.nextDown)
+		QtCore.QObject.disconnect(self.ui.btnNext, QtCore.SIGNAL('released()'), self.nextUp)
+		QtCore.QObject.disconnect(self.ui.btnHate, QtCore.SIGNAL('pressed()'), self.hateDown)
+		QtCore.QObject.disconnect(self.ui.btnHate, QtCore.SIGNAL('released()'), self.hateUp)
+		QtCore.QObject.disconnect(self.ui.btnNext, QtCore.SIGNAL('clicked()'), self.playNextSong)
+		QtCore.QObject.disconnect(self.ui.btnHate, QtCore.SIGNAL('clicked()'), self.doHate)
+		QtCore.QObject.disconnect(self.ui.btnPlay, QtCore.SIGNAL('clicked()'), self.doPlayPause)
+		QtCore.QObject.disconnect(self.ui.btnLove, QtCore.SIGNAL('clicked()'), self.doLove)
+		QtCore.QObject.disconnect(self.ui.btnSleep, QtCore.SIGNAL('clicked()'), self.tiredSong)
+		QtCore.QObject.disconnect(self.player,QtCore.SIGNAL("EOF"), self.playNextSong)
+		QtCore.QObject.disconnect(self.player,QtCore.SIGNAL("state_change"), self.handleStates)
+		QtCore.QObject.disconnect(self.player,QtCore.SIGNAL("duration(int,int)"), self.setDuration)
+		QtCore.QObject.disconnect(self.s, QtCore.SIGNAL('shutdown'), self.closeEvent)
+		QtCore.QObject.disconnect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self.deleteStation)
+		QtCore.QObject.disconnect(self.ui.actionEnable_FM_Radio, QtCore.SIGNAL('triggered()'), self.doFM)
+		QtCore.QObject.disconnect(self.ui.actionSnooze, QtCore.SIGNAL('triggered()'), self.setSnooze)
+		QtCore.QObject.disconnect(self.ui.actionCreate, QtCore.SIGNAL('triggered()'), self.searchStation)
+		QtCore.QObject.disconnect(self.ui.actionDonate, QtCore.SIGNAL('triggered()'), self.open_donate)
+		QtCore.QObject.disconnect(self.ui.actionAdd_Seed, QtCore.SIGNAL('triggered()'), self.doSeed)
+		QtCore.QObject.disconnect(self.ui.actionSettings, QtCore.SIGNAL('triggered()'), self.openSettings)
+
+#Dialog Windows
+
+	def searchStation( self ): #Initiate station search
+		self.d = QDialog(self)
+		self.d.setParent(self)
+		vbox = QVBoxLayout()
+		hLayout = QHBoxLayout()
+		hLayout2 = QHBoxLayout()
+		hLayout3 = QHBoxLayout()
+		self.cStation = {}
+		lblSearch = QLabel('Search:')
+		self.cStation['lineSearch'] = QLineEdit()
+		self.cStation['radioArtist'] = QRadioButton('Artist')
+		self.cStation['radioSong'] = QRadioButton('Song')
+		btnGroup = QButtonGroup()
+		btnGroup.addButton(self.cStation['radioArtist'])
+		btnGroup.addButton(self.cStation['radioSong'])
+		self.cStation['radioArtist'].setChecked(1)
+		search = QPushButton("Search")
+		cancel = QPushButton("Cancel")
+		search.clicked.connect(self.selectStation)
+		cancel.clicked.connect(self.d.close)
+		hLayout.addWidget(lblSearch)
+		hLayout.addWidget(self.cStation['lineSearch'])
+		hLayout2.addWidget(self.cStation['radioArtist'])
+		hLayout2.addWidget(self.cStation['radioSong'])
+		hLayout3.addWidget(search)
+		hLayout3.addWidget(cancel)
+		vbox.addLayout(hLayout)
+		vbox.addLayout(hLayout2)
+		vbox.addLayout(hLayout3)
+		self.d.setLayout(vbox)
+		self.d.setWindowTitle("Create a Station:")
+		self.d.show()
+
+	def selectStation( self ): #Select a station search result.
+		self.d.close()
+		self.d = QDialog(self)
+		title = None
+		vbox = QVBoxLayout()
+		hLayout = QHBoxLayout()
+		hLayout2 = QHBoxLayout()
+		ret = self.player.search(self.cStation['lineSearch'].text())
+		if self.cStation['radioArtist'].isChecked():
+			if len(ret["artists"])>0:
+				songs = ret["artists"]
+				title = "artistName"
+		elif self.cStation['radioSong'].isChecked():
+			if len(ret["songs"])>0:
+				songs = ret["songs"]
+				title = "songTitle"
+		accept = QPushButton('Accept')
+		cancel = QPushButton('Cancel')
+		if not title:
+			self.cStation['stationList'] = QLineEdit("No Stations Found")
+			accept.setDisabled(True)
+			self.cStation['stationList'].setDisabled(True)
+		else:
+			self.cStation['stationList'] = QListWidget()
+			self.cStation['songs'] = songs
+			for s in songs:
+				self.cStation['stationList'].addItem(s[title])
+			#end for
+		accept.clicked.connect(self.createStation)
+		cancel.clicked.connect(self.d.close)
+		hLayout.addWidget(self.cStation['stationList'])
+		hLayout2.addWidget(accept)
+		hLayout2.addWidget(cancel)
+		vbox.addLayout(hLayout)
+		vbox.addLayout(hLayout2)
+		self.d.setLayout(vbox)
+		self.d.setWindowTitle("Select Station:")
+		self.d.show()
+
+	def createStation( self ): #Create selected station.
+		self.d.close()
+		ret = self.player.createStation( self.cStation['songs'][self.cStation['stationList'].currentRow()]['musicId'] )
+		print ret
+		self.autoPlay = ret
+		self.doPlayPause()
+		self.getStations()
+		self.cStation = None
+
+	def doSeed( self ): # Create Seed Window
+		self.d = QDialog(self)
+		self.d.setParent(self)
+		vbox = QVBoxLayout()
+		hLayout = QHBoxLayout()
+		m = QPushButton("Seed Artist")
+		p = QPushButton("Seed Song")
+		c = QPushButton("Close")
+		hLayout.addWidget(m)
+		hLayout.addWidget(p)
+		hLayout.addWidget(c)
+		m.clicked.connect(lambda: self.addSeed("artist"))
+		p.clicked.connect(lambda: self.addSeed("song"))
+		c.clicked.connect(self.d.close)
+		vbox.addLayout(hLayout)
+		self.d.setLayout(vbox)
+		self.d.setWindowTitle("Add Station Seed:")
+		self.d.show()
+
+	def addSeed( self, seed=None ): #Seed target and Close Seed Window
+		if seed == 'artist':
+			musicId = self.song[1]['artistMusicId']
+		elif seed == 'song':
+			musicId = self.song[1]['musicId']
+		else:
+			return
+		self.stimer.singleShot(100, lambda: self.player.doSeed( musicId, self.curStation ))
+		self.d.close()
+
+	def open_donate( self ): #Donations are AWESOME :)
+		self.d = QDialog(self)
+		self.d.setParent(self)
+		vbox = QVBoxLayout()
+		hLayout = QHBoxLayout()
+		m = QPushButton("pyRadio")
+		p = QPushButton("Get Pandora One") #Let's support our radio host :)
+		c = QPushButton("Close")
+		hLayout.addWidget(m)
+		hLayout.addWidget(p)
+		hLayout.addWidget(c)
+		action = lambda: self.donate('radio')
+		action1 = lambda: self.donate('pandora')
+		m.clicked.connect(action)
+		p.clicked.connect(action1)
+		c.clicked.connect(self.d.close)
+		vbox.addLayout(hLayout)
+		self.d.setLayout(vbox)
+		self.d.setWindowTitle("Donate to:")
+		self.d.show()
+
+	def donate( self, who=None ): #Open browser and close Donation window
+		if who == 'radio':		
+			webbrowser.open_new('https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=BWX2M54JBJ2T2')
+		elif who == 'pandora':
+			webbrowser.open_new('http://www.pandora.com/pandora_one')
+		self.d.close()
+
 	def openSettings( self ): #Build and open settings dialog box.
 		self.d = QDialog(self)
 		self.d.setParent(self)
 		self.settings = {} #Store settings in a hash, it's easier to read in my opinion.
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
 		vbox = QVBoxLayout()
 		hLayout = QHBoxLayout()
 		hLayout2 = QHBoxLayout()
@@ -268,7 +606,7 @@ class Panda(QtGui.QMainWindow): #main class
 		self.d.setWindowTitle("pyRadio Settings:")
 		self.d.show()
 
-	def setSettings ( self ): #Set settings.
+	def setSettings ( self ): #Set settings and close settings dialog.
 		changed = False
 		if self.c.user != self.settings['lineUser'].text() or self.c.password != self.settings['linePass'].text():
 			self.c.user = self.settings['lineUser'].text()
@@ -283,9 +621,8 @@ class Panda(QtGui.QMainWindow): #main class
 			format = 'aacplus'
 		if format != self.c.format:
 			self.c.format=format
-			self.player.format=format
 			self.player.playlist = []
-			self.player.getPlaylist = 1 #Get a new playlist with the new format!
+			self.player.getMoreSongs( self.curStation, self.c.format )
 			changed = True
 		if self.os == "RX-51":
 			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
@@ -302,355 +639,7 @@ class Panda(QtGui.QMainWindow): #main class
 			self.c.write()
 		self.d.close()
 
-	def doSeed( self ):
-		if not self.player.playing:
-			QtGui.QMessageBox.warning(self, "Not Playing", "You must be listening to a song to seed it.")
-			return
-
-		self.d = QDialog(self)
-		self.d.setParent(self)
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		vbox = QVBoxLayout()
-		hLayout = QHBoxLayout()
-		m = QPushButton("Seed Artist")
-		p = QPushButton("Seed Song")
-		c = QPushButton("Close")
-		hLayout.addWidget(m)
-		hLayout.addWidget(p)
-		hLayout.addWidget(c)
-		m.clicked.connect(lambda: self.addSeed("artist"))
-		p.clicked.connect(lambda: self.addSeed("song"))
-		c.clicked.connect(self.d.close)
-		vbox.addLayout(hLayout)
-		self.d.setLayout(vbox)
-		self.d.setWindowTitle("Add Station Seed:")
-		self.d.show()
-
-	def addSeed( self, seed=None ):
-		if self.player.playing:
-			self.player.seedSong = seed
-		self.d.close()
-		if self.os == "RX-51":
-			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
- 
-	def open_donate( self ): #Donations are AWESOME :)
-		self.d = QDialog(self)
-		self.d.setParent(self)
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.d.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.d.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		vbox = QVBoxLayout()
-		hLayout = QHBoxLayout()
-		m = QPushButton("pyRadio")
-		p = QPushButton("Get Pandora One") #Let's support our radio host :)
-		c = QPushButton("Close")
-		hLayout.addWidget(m)
-		hLayout.addWidget(p)
-		hLayout.addWidget(c)
-		action = lambda: self.donate('radio')
-		action1 = lambda: self.donate('pandora')
-		m.clicked.connect(action)
-		p.clicked.connect(action1)
-		c.clicked.connect(self.d.close)
-		vbox.addLayout(hLayout)
-		self.d.setLayout(vbox)
-		self.d.setWindowTitle("Donate to:")
-		self.d.show()
-
-	def donate( self, who=None ):
-		if who == 'radio':		
-			webbrowser.open_new('https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=BWX2M54JBJ2T2')
-		elif who == 'pandora':
-			webbrowser.open_new('http://www.pandora.com/pandora_one')
-		self.d.close()
-
-	def stationCreated( self ): #Station Created Successfully
-		ok = QtGui.QMessageBox.warning(self,'Station Created','Station Created, your station list will reload.')
-		self.ui.listStations.clear()
-		self.player.getStatlist = 1
-	
-	def setDuration( self ): #Set the song position/duration
-		if (self.player.duration-self.player.position) < self.player.duration:
-			position = "%02d:%02d" % divmod(self.player.duration - self.player.position, 60) + " / " + "%02d:%02d" % divmod(self.player.duration, 60)
-			self.ui.lblTime.setText(position + "\nAlbum: " + self.player.curSong[1]["album"])
-
-	def createStation( self ): #Start the create station sequence
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		station, ok = QtGui.QInputDialog.getText(self,"Artist or Track Name", "Enter an Artist or Track Name:")
-		if len(station)>0:
-			self.ui.albumArt.setHtml("Please wait..... ")
-			ret = self.player.search(station)
-			if len(ret["artists"])>0 or len(ret["songs"])>0:
-				if len(ret["artists"])>0 and len(ret["songs"])>0:
-					what = QtGui.QMessageBox.question(self, 'Artist or Track?', 'Is this an artist or track name?','Artist','Track')
-					if what == 0:
-						songs = ret["artists"]
-						title = "artistName"
-					else:
-						songs = ret["songs"]
-						title = "songTitle"
-				else:
-					if len(ret["artists"])>0:
-						songs = ret["artists"]
-						title = "artistName"
-					else:
-						songs = ret["songs"]
-						title = "songTitle"
-
-				self.ui.listStations.clear() 
-				#For lack of room, populate station list with new stations.  I want to change this to be it's own QDialog in the future.
-				for s in songs:
-					self.ui.listStations.addItem(s[title])
-					self.createStations = songs
-			ok = QtGui.QMessageBox.warning(self,'Select Match','Select the closest title from the station drop down.')
-		if self.os == "RX-51":
-			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
-
-	def stationDeleted( self ): #Delete station
-		ok = QtGui.QMessageBox.warning(self, "Station Deleted", "Station Deleted.")
-
-	def sigTired( self ): #Song marked tired successfully 
-		ok = QtGui.QMessageBox.warning(self, "Tired", "Song marked tired.")
-		self.playNextSong()
-
-	def deleteStation(self): #Start the delete station process
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		reply = QtGui.QMessageBox.question(self, 'Delete Station',
-			"Really delete "+self.ui.listStations.currentText()+"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
-		if reply == QtGui.QMessageBox.Yes:
-			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
-			self.player.stopPlayBack()
-			self.player.deleteStation = 1
-			self.ui.albumArt.setHtml("Please wait...")
-		if self.os == "RX-51":
-			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
-
-	def tiredSong(self): #Start tired song process and wait for signal from radio backend
-		check = QtGui.QMessageBox.question(self, 'Are you sure?', 'Are you tired of the song ' + self.player.curSong[1]["title"] + '?','No','Yes')
-		if check == 1:
-			self.player.tiredSong = 1
-
-	def setSnooze(self): #Set tired
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		ktime,ok = QtGui.QInputDialog.getText(self,"Snooze", "Enter a time, in minutes, to shutdown:")
-		if ok:
-			self.s.cont = 1
-			self.s.ktime = int(ktime)
-			self.s.start()
-		if self.os == "RX-51":
-			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
-
-	def doFM(self): #Turn on FM transmitter
-		state = self.fmtx_iface.Get("com.nokia.FMTx.Device", "state")
-		if state == 'disabled':
-			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Disable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
-			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'enabled', variant_level=1))
-		else:
-			self.ui.actionEnable_FM_Radio.setText(QtGui.QApplication.translate("pyRadio", "Enable FM Tx", None, QtGui.QApplication.UnicodeUTF8))
-			self.fmtx_iface.Set("com.nokia.FMTx.Device", "state", dbus.String(u'%s' % 'disabled', variant_level=1))
-
-	def nextDown(self): #change button color
-		self.ui.btnNext.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")
-
-	def nextUp(self): #change button color
-		self.ui.btnNext.setStyleSheet("background-color: rgb(0, 0, 0);\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/next_small.png);")
-
-	def doPlayPause( self, signal=None ): #Play if stopped, pause if playing, receive signals from Backend if changed from somewhere else (ie: headphones)
-		if self.player.playing:
-			if signal != "play":
-				self.player.stopPlayBack('pause')
-				self.player.playing = False
-				self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
-		elif self.player.playlist != []:
-			if signal != "pause":
-				self.player.startPlayBack()
-				self.player.playing = True
-				self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
-		else:
-			self.playStation()
-			self.player.playing = True
-			self.ui.btnPlay.setStyleSheet("background-color: "+ self.c.color +";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/Play_small.png);")
-
-	def doLove( self ): #Love this song
-		self.player.loveHateRating = True
-		self.player.loveHateSong = 1
-		self.player.curSong[1]["rating"] = 1
-		self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
-		"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
-
-	def doHate( self ): #Hate this song
-		self.player.loveHateRating = False
-		self.player.loveHateSong = 1
-		self.skip = 1
-
-	def sigLoveHate( self ): #Signal for Love/Hate from backend
-		if not self.player.respLH:
-			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color +";\n"
-			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
-		if self.skip:
-			self.playNextSong()
-
-	def hateDown(self): #change button color
-		self.ui.btnHate.setStyleSheet("background-color: " + self.c.color + ";\nborder: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
-
-	def hateUp(self): #change button color
-		self.ui.btnHate.setStyleSheet("background-color: rgb(100, 100, 100);\nborder: rgb(255, 255, 255);\nbackground-image: url(:/img/img/thumbdwn_small.png);")
-
-	def sigAuth( self ): #Signal for authentication success/failure.
-		if self.player.success:
-			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication succeeded.  Wait for station list to appear in drop down below and select a station.</font></td><tr></table>" )
-			self.count = 0
-			self.player.getStatlist = 1
-		else:
-			self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authentication failed.  Please try again.</font></td><tr></table>" )
-			self.c.user = ""
-			self.c.password = ""
-			self.auth()
-
-	def auth( self ): #Start authentication process.
-		if self.os == "RX-51":
-			if not self.portrait:
-				self.setAttribute(Qt.WA_Maemo5LandscapeOrientation, 1)
-			else:
-				self.setAttribute(Qt.WA_Maemo5PortraitOrientation, 1)
-		if self.count>2:
-			self.closeEvent()
-		self.count += 1
-		ok = True #In case we got the user/pass from config
-		if self.c.user == "":
-			self.c.user,ok = QtGui.QInputDialog.getText(self,"Username", "Pandora Username:", QLineEdit.Normal)
-		if not ok: #Username was cancelled.. they must be done.
-			self.closeEvent()
-		if self.c.password == "": 
-			self.c.password,ok = QtGui.QInputDialog.getText(self,"Password", "Pandora Password:", QLineEdit.Password)
-		if not ok: #Password was cancelled.. they must be done.
-			self.closeEvent()
-		self.ui.albumArt.setHtml( "<table width='100%' height='100%'><tr height='100%'><td height='100%' valign='middle'><font style='color: #fff'>Authenticating to Pandora as "+self.c.user+" ... </font></td><tr></table>" )
-		self.player.user = self.c.user
-		self.player.password = self.c.password
-		self.player.doAuth = 1
-		self.authenticated = True
-		self.c.write()
-		if self.os == "RX-51":
-			self.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
-
-	def playStation( self ): #Play currently selected station
-		if len(self.createStations)>0: #If we're creating a station, finish creating and not playing selected station
-			self.player.createStationId = self.createStations[self.ui.listStations.currentIndex()]['musicId']
-			self.createStations = []
-			return
-		
-		if self.player.playStation > 0:
-			stationId = self.player.playStation
-			self.player.playStation = 0
-		else:
-			stationId = self.player.stations[self.ui.listStations.currentIndex()]['stationId']	
-		self.player.curStation = stationId
-		self.player.playlist = []
-		self.player.getMoreSongs()
-		self.playNextSong()
-		if stationId != self.c.lastplayed:
-			f = open(os.path.expanduser("~")+'/.config/pyradio/lastplayed', 'w')
-			f.write(self.player.curStation)
-			f.close()
-
-	def getStations( self ): #Get Station List	
-		self.count = 0
-		self.ui.listStations.clear()
-		self.ui.btnPlay.setEnabled( True )
-		self.ui.listStations.setEnabled( True )
-		self.ui.btnLove.setEnabled( True )
-		self.ui.btnHate.setEnabled( True )
-		self.ui.btnSleep.setEnabled( True )
-		self.ui.btnNext.setEnabled( True )
-		self.ui.lblTime.setWordWrap( True )
-
-		self.buildPlaylist()
-
-		if self.player.playStation > 0:
-			self.playStation()
-
-	def buildPlaylist( self ): #Populate the station list box
-		for s in self.player.stations: #Stations need to be scanned for QuickMix seperately
-			if s["isQuickMix"]:
-				qMixIds = s["quickMixStationIds"]
-				break
-
-		for s in self.player.stations: #Add stations
-			#print s
-			#print ""
-			sName = s["stationName"]
-			for qMixId in qMixIds:
-				if qMixId == s["stationId"]:
-					sName += " *" #If we're in the QuickMix list we add a *
-			self.ui.listStations.addItem(sName)
-			if self.player.playStation>0 and s["stationId"] == self.player.playStation:
-				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
-	
-			if self.player.curStation>0 and s["stationId"] == self.player.curStation:
-				self.ui.listStations.setCurrentIndex(self.ui.listStations.count()-1)	
-
-	def setUserPass( self ): #Reset username/password and start authenticating
-		self.c.user = ""
-		self.c.password = ""
-		self.auth()
-
-	def playNextSong( self ): #Go to next song in playlist
-		try:
-			next = self.player.playlist.pop( 0 )
-		except:
-			self.player.getMoreSongs()
-			next = self.player.playlist.pop( 0 )
-
-		print "Playing Next Song"
-		self.skip = 0
-		self.player.curSong = next
-		self.player.playSong ( next )
-		self.setUi ( next )
-		if self.player.playlist == []:
-			#Out of songs, grab some more while playing
-			self.player.getPlaylist = 1
-	
-	def setUi ( self, next=None ): #Update the album graphic and set Love graphic
-		art = next[1]["art"]
-		self.ui.albumArt.setHtml("<table width='100%' height='100%'><tr><td vertical-align='middle' style='text-align: center'><img style='width: 80%; height: 100%' src='"+art+"'></td></tr></table>")
-		self.ui.lblSong.setText("\"" + next[1]["title"] + "\"\nArtist: " + next[1]["artist"])
-		if next[1]["rating"] > 0:
-			self.ui.btnLove.setStyleSheet("background-color: "+ self.c.color + ";\n"
-			"border: rgb(0, 0, 0);\nbackground-image: url(:/img/img/thumbup_small.png);")
-		else:
-			self.ui.btnLove.setStyleSheet("background-image: url(:/img/img/thumbup_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: rgb(100, 100, 100);")
-
-		self.ui.btnPlay.setStyleSheet("background-image: url(:/img/img/Play_small.png);\nborder: rgb(0, 0, 0);\nbackground-color: "+ self.c.color + ";")
-		self.ui.btnNext.setEnabled( True )
-
-	def stop( self ): #Stop Playing, not currently implemented
-		self.player.playing = False
-
-	def closeEvent(self, event=None): #Exit application gracefully.. somewhat.
-		self.player.run = 0
-		self.player.close()
-		del self.ui
-		del self.player
-		quit()
+#Main Method
 
 if __name__ == '__main__':
 	if not ( os.path.exists( os.path.dirname(sys.argv[0])+"/libpiano/crypt_key_input.h" ) \
